   /************************************************************/
  /*****            Beacon Controller Program             *****/
 /*****     (c)2003 Stavros S. Fotoglou - SV6GMP         *****/
/************************************************************/

//++++++++++++++++++++++  Device AT90S2313 ++++++++++++++++++++++
#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>
#include <progmem.h>

#include "morse.h"

#define TIME            5               //Minutes
//#define WPM             20
#define CPM             70              //Characters per minute
#define INT_PER         .000125         //Sample frequency 8000Hz
//#define DPS             (WPM/1.2)
#define DPS             (CPM/6)
#define EL_DELAY        (1/(DPS*INT_PER))
#define LOOP_DELAY      (TIME*60)/(256*INT_PER)

#define IC_ADDR         0xa0

//***** PORT D pins definition *****
#define SCL             (1<<0)          //PD0 pin 2
#define SDA             (1<<1)          //PD1 pin 3
#define TST             (1<<2)          //PD2 pin 6
#define PTT             (1<<3)          //PD3 pin 7
#define PL1             (1<<4)          //PD4 pin 8
#define PL2             (1<<5)          //PD5 pin 9
#define PL3             (1<<6)          //PD6 pin 11

//***** flag bit definition *****
#define PLAY            (1<<0)


//typedef unsigned char u08;
//typedef unsigned short u16;

//***** IIC base routines *****
void ic_start(void);
void ic_stop(void);
void ic_write(u08);
u08 ic_read(void);
void ic_get_ack(void);
void ic_put_ack(void);
void ic_scl_extra(void);

//***** High level routines *****
void delay1(u08);
u08 eeprom_read(u16);
void play_clip(u16, u16);
void say(u08);
void program(void);

void ptt_on(void);
void ptt_off(void);
void pl(u08);

//***** Morse code routines *****
void tone(void);
void teleia(void);
void pavla(void);
void space(void);
void word_spc(void);
void keying_decode(u08);
void keying_char(char);
void keying_string(char *);

u16 __attribute__ ((progmem)) words[] = {0, 24093,      //0 - sv6gmp
                                         24207, 29732,  //1 - beacon
                                         29846, 36083,  //2 - preveza
                                         36084, 41720,  //3 - deka (10)
                                         41721, 46563,  //4 - eikosi (20)
                                         46592, 51946,  //5 - triada (3)0
                                         52000, 60973}; //6 - dbm

char __attribute__ ((progmem)) morse_id[] = "SV6GMP BEACON QTH PREVEZA ";
char __attribute__ ((progmem)) level[] = "POWER LEVEL ";
char __attribute__ ((progmem)) triada[] = "30 DBM";
char __attribute__ ((progmem)) eikosi[] = "20 DBM";
char __attribute__ ((progmem)) deka[] = "10 DBM";

u08 p, flag, cnt1;
u16 add, stop, beep_duration, counter, timer;

SIGNAL(SIG_OVERFLOW0)
{
static u08 s;
u08 sample;
outp(105, TCNT0);
++cnt1;
--counter;
if (!cnt1)
    --timer;            //decrease every 31.25 Hz = .032sec
s=inp(PORTD);
if (s & TST)
   s = s & (~TST);
else
   s = s | TST;
outp(s, PORTD);

if(flag & PLAY)
  {
  ic_put_ack();
  sample = ic_read();
  outp(sample, PORTB);
  ++add;
  if(add > stop)
     flag &= (~PLAY);
  }
if(beep_duration)
   {
    tone();
    --beep_duration;
   }
}

int main(void)
{
outp(0xfc, DDRD);       //Port D D0,D1 are inputs
outp(0xfc, PORTD);      //Pin D0,D1 hi-z inputs
outp(0xff, DDRB);       //Port B is output
outp(0xff, PORTB);
flag = cnt1 = beep_duration = timer = 0;
outp((1<<TOIE0), TIMSK);
outp(2, TCCR0);
outp(100, TCNT0);

p=inp(DDRD);

sei();

//play_clip(0, 65534);
//play_clip(36083, 41722);
//program();
for(;;)
   {
    if(!timer)
      {
       program();
       timer = LOOP_DELAY;
      }
    delay1(25);
   }
}

void program(void)
{
//****************** 30 dbm **************
pl(~4);
ptt_on();
word_spc();
say(0);         //sv6gmp
say(1);         //beacon
say(2);         //preveza
say(5);         //30
say(6);         //dbm
say(1);         //beacon
keying_string((char *)morse_id);
keying_string((char *)level);
keying_string((char *)triada);
ptt_off();
word_spc();
//****************** 20 dbm ***************
pl(~2);
ptt_on();
word_spc();
say(0);         //sv6gmp
say(1);         //beacon
say(2);         //preveza
say(4);         //20
say(6);         //dbm
say(1);         //beacon
keying_string((char *)morse_id);
keying_string((char *)level);
keying_string((char *)eikosi);
ptt_off();
word_spc();
//****************** 10 dbm *****************
pl(~1);
ptt_on();
word_spc();
say(0);         //sv6gmp
say(1);         //beacon
say(2);         //preveza
say(3);         //10
say(6);         //dbm
say(1);         //beacon
keying_string((char *)morse_id);
keying_string((char *)level);
keying_string((char *)deka);
ptt_off();
}

void pl(u08 x)
{
u08 pd;
x = x << 4;
x = x & 0x70;
pd = inp(PORTD);
pd = pd & 0x0f;
pd = pd | x;
outp(pd, PORTD);
}

void ptt_on(void)
{
u08 pd;
pd = inp(PORTD);
pd = pd | PTT;
outp(pd, PORTD);
}

void ptt_off(void)
{
u08 pd;
pd = inp(PORTD);
pd = pd & (~PTT);
outp(pd, PORTD);
}

void keying_string(char *morse)
{
u08 i=0;
char a;
while((a=PRG_RDB(&morse[i])) !='\0')
     {
     keying_char(a);
     ++i;
     }
}

void keying_char(char c)
{
u08 a, ch;
if (c == ' ')
   {
    word_spc();
   }
else
   {
   if ((c >= 'A') && (c <= 'Z'))
       a = c - 65;
   else if((c >= '0') && (c <= '9'))
       a= c-22;
   else
       a=0;
   ch = PRG_RDB(&morse_chars[a]);
   keying_decode(ch);
   }
}

void keying_decode(u08 chr)
{
u08 i, len;
len= chr >> 5;
len = len & 7;
chr = chr << 3;
for (i=0; i < len; ++i)
    {
    if(chr & 128)
       pavla();
    else
       teleia();
    chr=chr<<1;
    space();
    }
space();
space();
}

void teleia(void)
{
beep_duration = EL_DELAY;
do {
    delay1(5);
   }while(beep_duration);

}

void pavla(void)
{
u08 i;
for(i=0; i<=2; ++i)
    teleia();
}

void space(void)
{
counter = EL_DELAY;
do{
  delay1(5);
  }while(counter > 0);
outp(0x80, PORTB);
}

void word_spc(void)
{
u08 i;
for (i=0; i<6; ++i)
     space();
}

void tone(void)
{
if(cnt1 & 8)
   outp(0x80, PORTB);
else
   outp(0x00, PORTB);
}

void say(u08 idx)
{
u16 clip_start, clip_end;
u08 *a;
a = (u08*) &words;
a = a + (idx << 2);
clip_start = (PRG_RDB(a + 1) << 8) | (PRG_RDB(a));
clip_end = (PRG_RDB(a + 3) << 8) | (PRG_RDB(a + 2));
play_clip(clip_start, clip_end);
}

void play_clip(u16 start, u16 end)
{
u08 sample;
p=inp(DDRD);
add = start;
stop = end;
sample=eeprom_read(add);
outp(sample,PORTB);
flag |= PLAY;
do {
 delay1(50);
   } while(flag & PLAY);
outp(0x80, PORTB);
ic_stop();
}

u08 eeprom_read(u16 address)
{
u08 addh, addl, data;   //p
addl=address & 0x00ff;
address = address >> 8;
addh = address;

ic_start();
ic_write(IC_ADDR);
ic_get_ack();

ic_write(addh);
ic_get_ack();

ic_write(addl);
ic_get_ack();

//p=inp(DDRD);
p=p|SCL;
outp(p,DDRD);
delay1(3);
p=p&(~SCL);
outp(p,DDRD);
delay1(3);

ic_start();
ic_write(IC_ADDR + 1);
ic_get_ack();

data=ic_read();
//ic_stop();
return(data);
}

void ic_start(void)
{
//u08 p;
//p = inp(DDRD);
p = p | SDA;            //Pin SDA low while SCL is hi
outp(p, DDRD);
delay1(3);
}

void ic_write(u08 data)
{
u08 i;   //p
//p = inp(DDRD);

for(i = 0; i < 8; ++i)
    {
     p = p | SCL;            //Pin SCL low
     outp(p, DDRD);
     if (data & 128)         //Is MSB 1?
         p = p & (~SDA);     //Pull up hi SDA
     else
         p = p | SDA;
     outp(p, DDRD);

     delay1(3);

     p = p & (~SCL);         //Pin SCL hi
     outp (p, DDRD);

     delay1(3);

     data = data << 1;
     
    }
}

void ic_get_ack(void)
{
u08 a;   //p
//p = inp(DDRD);
p = p | SCL;            //SCL low
p = p & (~SDA);         //SDA hi-Z
outp (p, DDRD);
delay1(3);
p = p & (~SCL);         //SCL hi
outp (p, DDRD);
do {
   a = inp(PIND);
   } while(a & SDA);
delay1(3);
}

void ic_put_ack(void)
{
//u08 p;
//p =inp(DDRD);
p=p | SCL;
outp(p, DDRD);
p=p | SDA;
outp(p, DDRD);
delay1(3);
p=p & (~SCL);
outp(p, DDRD);
delay1(3);
//p=p & (~SDA);
//outp(p, DDRD);
}

u08 ic_read(void)
{
u08 i, a, b, res; //p
res=0;
b=128;

//p = inp(DDRD);

for(i = 0; i < 8; ++i)
    {
     p = p | SCL;               //Pin SCL low
     p = p & (~SDA);
     outp(p, DDRD);
     delay1(3);
     p = p & (~SCL);            //Pin SCL hi
     outp(p, DDRD);
     delay1(3);
     a = inp(PIND);
     a = a & SDA;
     if (a)
          res=res+b;
     b=b>>1;
    }
return(res);
}

void ic_stop(void)
{
//u08 p;
//p = inp(DDRD);
p = p | SDA | SCL;      //SCL , SDA low
outp(p, DDRD);
delay1(3);
p = p & (~SCL);         //SCL hi while SDA is low
outp(p, DDRD);
delay1(3);
p = p & (~SDA);         //SDA hi
outp(p, DDRD);
delay1(3);
}

void ic_scl_extra(void)
{
p=p|SCL;
outp(p,DDRD);
delay1(3);
p=p&(~SCL);
outp(p,DDRD);
delay1(3);
}

void delay1(u08 imax)
{
u08 i, n;
for(i=0; i<imax; ++i)
    ++n;
}


